// TemplateProj.cpp : Template Direct3D project
//

#include <windows.h>
#include "d3d9.h"
#include "d3dx9.h"
#include "..\..\common\dxfunc.h"

// Global Variables:
char szWindowClass[] = "Direct3DTemplateProj";
char szTitle[] = "Direct3D Demo by Michael Flenov";

// Direct3D objects
IDirect3D9 *pD3D = NULL;
IDirect3DDevice9 *pD3DDevice = NULL;

int iWidth=800;
int iHeight=600; 

float ViewAngle=0;

DWORD dwNumMaterials;
ID3DXMesh *pMesh;
LPDIRECT3DTEXTURE9 *pMeshTextures;
D3DMATERIAL9 *pMeshMaterials;

const char DIR[16] = "../../../media/";

// Forward declarations of functions included in this code module:
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int nCmdShow);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

bool Init(HWND hWnd);
void GraphEngine();

int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int nCmdShow)
{
  WNDCLASSEX wcex;
  MSG        msg;
  HWND       hWnd;

  CoInitialize(NULL);

  // Register window class
  wcex.cbSize        = sizeof(wcex);
  wcex.style         = CS_CLASSDC;
  wcex.lpfnWndProc   = (WNDPROC)WndProc;
  wcex.cbClsExtra    = 0;
  wcex.cbWndExtra    = 0;
  wcex.hInstance     = hInst;
  wcex.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wcex.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wcex.hbrBackground = NULL;
  wcex.lpszMenuName  = NULL;
  wcex.lpszClassName = szWindowClass;
  wcex.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);
  if(!RegisterClassEx(&wcex))
    return FALSE;

  // Create the main window
  hWnd = CreateWindow(szWindowClass, szTitle, WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
              CW_USEDEFAULT, CW_USEDEFAULT, iWidth, iHeight, NULL, NULL, hInst, NULL);
  if(!hWnd)
    return FALSE;
  ShowWindow(hWnd, SW_NORMAL);
  UpdateWindow(hWnd);

  if(Init(hWnd) == TRUE) 
  {
	  while (true) 
	  {
		  if (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) 
		  {
			  TranslateMessage(&msg);
			  DispatchMessage(&msg);
			  if (msg.message == WM_QUIT) break;
		  }
		  GraphEngine();
	  }
  }

  if (pD3DDevice) {pD3DDevice= NULL; pD3DDevice=NULL;}
  if (pD3D) {pD3D= NULL; pD3D=NULL;}

  CoUninitialize();

  return 0;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch(message) {
    case WM_DESTROY:
      PostQuitMessage(0);
      break;

    default:
      return DefWindowProc(hWnd, message, wParam, lParam);
  }

  return 0;
}

bool Init(HWND hWnd)
{
  if (DX3DInitZ(&pD3D, &pD3DDevice, hWnd, iWidth, iHeight, FALSE)!=S_OK)
  {
	  MessageBox(hWnd, "DirectX Initialize Error", "Error", MB_OK);
	  return FALSE;
  }
  
  SetCurrentDirectory(DIR);
  dwNumMaterials = LoadMesh("tiny.x", pD3DDevice,
			   &pMesh, &pMeshTextures,
			   "texture.bmp", &pMeshMaterials
			   );
  
  D3DXMATRIXA16 mat;
  D3DXMatrixIdentity(&mat);
  D3DXMatrixRotationZ(&mat, 90);
  pD3DDevice->SetTransform(D3DTS_VIEW, &mat);

  D3DLIGHT9 light;
  ZeroMemory(&light, sizeof(D3DLIGHT9));
  light.Type = D3DLIGHT_DIRECTIONAL;
  light.Direction = D3DXVECTOR3(0.5f, 0.0f, 0.5f);
  light.Diffuse.r = light.Diffuse.g = light.Diffuse.b = light.Diffuse.a = 1.0f;
  pD3DDevice->SetLight(0, &light);
  pD3DDevice->LightEnable(0, TRUE);

  return TRUE;
}

void GraphEngine()
{
  pD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(255,255,255), 1.0f, 0);
  if (SUCCEEDED(pD3DDevice->BeginScene())) 
  {
	  ViewAngle+=0.01f;
	  float b = ViewAngle;
      
	  D3DMATRIX World = {
		cos(b)*cos(b), cos(b)*sin(b), sin(b), 0,
		-sin(b), cos(b), 0, 0,
		-sin(b)*cos(b), -sin(b)*sin(b), cos(b), 0,
		0, 0,  700, 1,
	  };
	  
	  pD3DDevice->SetTransform(D3DTS_WORLD, &World);
	  pD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);

	  for (DWORD i=0; i<dwNumMaterials; i++)
	  {
		  pD3DDevice->SetMaterial(&pMeshMaterials[i]);
		  if (pMeshTextures[i])
			  pD3DDevice->SetTexture(0, pMeshTextures[i]);
		  pMesh->DrawSubset(i);
	  }		

	  pD3DDevice->EndScene();
  }
  
  pD3DDevice->Present(NULL, NULL, NULL, NULL);
}
