float4x4 view_proj_matrix;
float4x4 view_matrix;

float4 color =  {1, 0.3f, 1, 1};
float4 gloss=  {0.7f, 0.5f, 0.4f, 1};
float4 lightDir =  {-0.4f, 0.4f, -0.8f, 0};
sampler Noise;

struct VS_INPUT 
{
  float4 Pos: POSITION;
  float3 normal: NORMAL;
  float3 tangent: TANGENT;
};

struct VS_OUTPUT {
   float4 Pos:      POSITION;
   float3 normal:   TEXCOORD0;
   float3 tangent:  TEXCOORD1;
   float3 viewVec:  TEXCOORD3;
   float3 pos:      TEXCOORD4;
};

VS_OUTPUT vertsh(VS_INPUT In)
{
   VS_OUTPUT Out;

   Out.Pos = mul(In.Pos, view_proj_matrix);

   Out.normal   =  mul(In.normal, view_matrix);
   Out.tangent  =  mul(In.tangent, view_matrix);

   Out.viewVec  = normalize(-mul(In.Pos, view_matrix));
   Out.pos = In.Pos.xyz;

   return Out;
}


struct PS_INPUT 
{
  float3 normal: TEXCOORD0;
  float3 tangent: TEXCOORD1; 
  float3 viewVec: TEXCOORD3; 
  float3 pos: TEXCOORD4;
};

float4 pixsh(PS_INPUT In) : COLOR 
{
   float angle = (tex3D(Noise, In.pos) - 0.5);
   float cosA, sinA;
   sincos(angle, sinA, cosA);

   float3 tang = sinA * In.tangent + cosA * In.normal;

   float diffuse = saturate(dot(lightDir, In.normal));
   float cs = -dot(In.viewVec, tang);
   float sn = sqrt(1 - cs * cs);
   float cl = dot(lightDir, tang);
   float sl = sqrt(1 - cl * cl);
   float specular = pow(saturate(cs * cl + sn * sl), 32);

   return diffuse * color + gloss * specular;
}



technique PixelLight
{
    pass P0
    {
        VertexShader = compile vs_2_0 vertsh();
        PixelShader  = compile ps_2_0 pixsh();
    }
}