#include "TexFilter.h"

struct __declspec(uuid("{17B39DA6-A3DB-4312-81D4-F57308171C52}")) CLSID_AnimatedTexture;

CTexFilter::CTexFilter(IDirect3DDevice9 *pD3DDevice) 
  : CBaseVideoRenderer(__uuidof(CLSID_AnimatedTexture), NAME("ANIMATEDTEXTURE"), NULL, NULL)
{
	pDevice  = pD3DDevice;
}

CTexFilter::~CTexFilter()
{
}

HRESULT CTexFilter::CheckMediaType(const CMediaType *pmt)
{
    CheckPointer(pmt,E_POINTER);

	if(*pmt->FormatType() != FORMAT_VideoInfo)
		return E_INVALIDARG;
	
	if (IsEqualGUID(*pmt->Type(), MEDIATYPE_Video))
		if (IsEqualGUID(*pmt->Subtype(), MEDIASUBTYPE_RGB24))
			return S_OK;
	
	return E_FAIL;
}

HRESULT CTexFilter::SetMediaType(const CMediaType *pmt)
{
	D3DSURFACE_DESC ddsd;
	
	VIDEOINFO *pviBmp = (VIDEOINFO *)pmt->Format();	
	lVidWidth  = pviBmp->bmiHeader.biWidth;
	lVidHeight = abs(pviBmp->bmiHeader.biHeight);
	lVidPitch  = (lVidWidth * 3 + 3) & ~(3);
	
    if (FAILED(pDevice->CreateTexture(lVidWidth, lVidHeight, 1, 0, 
		D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &pTexture, NULL)))
		return S_FALSE;
	
	if (FAILED(pTexture->GetLevelDesc(0, &ddsd)))
		return S_FALSE;
	
	Format = ddsd.Format;
	if (Format != D3DFMT_A8R8G8B8)
		return VFW_E_TYPE_NOT_ACCEPTED;
	
	return S_OK;
}

HRESULT CTexFilter::DoRenderSample(IMediaSample *pMediaSample)
{
	BYTE *pBmpBuffer, *pTxtBuffer;
	pMediaSample->GetPointer(&pBmpBuffer);
 	
	D3DLOCKED_RECT d3dlr;
	if(FAILED(pTexture->LockRect(0, &d3dlr, 0, 0)))
		return E_FAIL;
	
	pTxtBuffer =  static_cast<byte *>(d3dlr.pBits);
	LONG lTxtPitch = d3dlr.Pitch;
	
    BYTE  * pbS = NULL;
    DWORD * pdwS = NULL;
    DWORD * pdwD = NULL;
	DWORD col, row;

	if (Format == D3DFMT_A8R8G8B8)
	{
        DWORD dwordWidth = lVidWidth / 4;                                       

        for (row = 0; row< (UINT)lVidHeight; row++)
        {
            pdwS = (DWORD*)pBmpBuffer;
            pdwD = (DWORD*)pTxtBuffer;

            for (col = 0; col < dwordWidth; col ++ )
            {
                pdwD[0] =  pdwS[0] | 0xFF000000;
                pdwD[1] = ((pdwS[1]<<8)  | 0xFF000000) | (pdwS[0]>>24);
                pdwD[2] = ((pdwS[2]<<16) | 0xFF000000) | (pdwS[1]>>16);
                pdwD[3] = 0xFF000000 | (pdwS[2]>>8);
                pdwD +=4;
                pdwS +=3;
            }

            pbS = (BYTE*) pdwS;
            for (col = 0; col < (UINT)lVidWidth % 4; col++)
            {
                *pdwD = 0xFF000000     |
                        (pbS[2] << 16) |
                        (pbS[1] <<  8) |
                        (pbS[0]);
                pdwD++;
                pbS += 3;           
            }

            pBmpBuffer  += lVidPitch;
            pTxtBuffer += lTxtPitch;
		}
	}
	
	if (FAILED(pTexture->UnlockRect(0)))
		return E_FAIL;
	
	return S_OK;
}
